//MATRIX.H  (Matrix Class)
//by Paul Landes (landes@jove.acs.unt.edu)
//6-8-96

// Copyright (C) 1997 by Paul Landes. This code/program is free 
// software; you can redistribute it and/or modify it under the terms 
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) 
// any later version. 

//This class dynamically creates a matrix of any type (T).  There are get
//and set functions that allow manipulation of data in the object.

#if !defined(_MATRIX_H)		//if not defined yet, define it.
#define _MATRIX_H

//___________________________class Matrix_________________________________
template <class T>
class Matrix
{
public:
  Matrix(int const, int const);
  virtual ~Matrix();

  //set a element in matrix
  int setElement(int const, int const, T const);
  //get an elemenet, parameters: row and col element, return val is type T
  int getElement(int const rowP, int const colP, T &result) const
  {
    if ( (rowP >= row) || (colP >= col) ) return 1;
    result = element[rowP][colP];
    return 0;
  }

  //get coordinate functions
  int getRow() const { return row; }
  int getCol() const { return col; }

private:
  T **element;		//double pointer allows 2d arrays
  int row, col;		//coordinates in matrix
};

//constructor allocates memeory with 'new' and sets all elements to 0
//(if includeing a class, you have to have overloaded = for this to work
template <class T>
Matrix<T>::Matrix (int const rowP, int const colP)
{
  int i, j;

  row = rowP;
  col = colP;

  element = new T * [row]; //allocate space for row type T pointers

  //fill each row of cols
  for (i = 0; i < row; i++) {
    element[i] = new T [col];		//allocate array of type T to each row
    for (j = 0; j < col; j++) element[i][j] = 0;
  }
}

//destructor first deletes all memory in cols for each row,
//then it deletes the pointer array
template <class T>
Matrix<T>::~Matrix()
{
  int i;
  for (i = 0; i < row; i++) delete [] element[i];
  delete [] element;
}

//set an element, if the row or col values are out of range, return
//exit failure.  Return: 0 on exit success, 1 on exit failure
template <class T>
int Matrix<T>::setElement(const int rowP, const int colP, const T value)
{
  if ( (rowP < row) && (colP < col) ) {
    element[rowP][colP] = value;
    return 0;
  }
  else return 1;
}

//this function that prints out a matrix
//CAUTION: this function prints row by row, col by col with cout so
//object passed must have cout << overloaded.
template <class T>
void printMatrix(Matrix<T> const &prMatrix)
{
  int rowcount = prMatrix.getRow();
  int colcount = prMatrix.getCol();
  int i, j;
  T tmpVar;

  //cell by cell print using cout
  for (i = 0; i < rowcount; i++) {
    for (j = 0; j < colcount; j++)
      {
    	prMatrix.getElement(i, j ,tmpVar);
	cout << setiosflags(ios::left) << setw(4) << tmpVar;
      }
    cout << endl << endl;
  }
}

#endif	//__MATRIX_H
